#!/bin/bash

# @author Bret Palsson <bret@getjive.com>
# @version 1.0 2009/10/10
#
# usage ./resize.sh [path to image]

#set -x
# Shrink and Truncate disk images
image=$1
loopback=`losetup -f`
looponly=`losetup -f | cut -d '/' -f3`
# Attach image file to a loop device
losetup $loopback $image

# List partions
fdisk -l $loopback
echo -n "Which partition are you resizing?: "
read partnumb

if [ -z "$partnumb" ]; then
echo "Which partition again?"
losetup -d $loopback
exit
fi

mapper=/dev/mapper/${looponly}p${partnumb}

# Create device maps from partition tables using the kpartx command: 
kpartx -a $loopback

# Get disk usage
mount $mapper /mnt/
df -h | grep 'mnt\|Size'
echo -n "How many Gigs should this partition be?: "
read size
if [ -z "$size" ]; then
echo "How many gigs did you want that partition?"
umount /mnt
kpartx -d $loopback
losetup -d $loopback
exit
fi
use=`df | grep $looponly | awk '{print $3}'`
current=`echo "scale=0; ($use/1024/1024)+2"|bc`
if [ "$size" -lt "$current" ]; then
echo "You are trying to make the disk smaller than its used capacity."
umount /mnt
kpartx -d $loopback
losetup -d $loopback
exit
fi
umount /mnt


# FSCK disk ~30minutes to complete
echo "This will take between 15 and 30 minutes: "
e2fsck -fytt $mapper

# Remove journal
tune2fs -O^has_journal $mapper

# Rechkeck disk -- Required before resize2fs Because we removed the journal
echo "This will take between 5 and 15 minutes: "
e2fsck -f $mapper

# Pre prepare disk by squishing all the data to beginning of file
# Please take note of the amount of blocks long and their size (ex. 4k).
echo "This will take about 30 minutes: "
newblock=$(resize2fs -p $mapper ${size}G | grep '^Resizing the filesystem' | sed -e 's/^Resizing.* \([[:digit:]]\+\) (\([[:digit:]]\+\)k) blocks\.$/\1 \2/')
b=`echo $newblock | awk '{print $1}'`
k=`echo $newblock | awk '{print $2}'`
let newsize=b*k

#resize2fs -p $mapper ${size}G

# Resize Disk
fdisk $loopback <<END
d
3
n
p
3

+${newsize}K
w
END

# Check device
fsck -n $mapper

# Enable Journal
tune2fs -j $mapper

# De-Mappify the loopback
kpartx -d $loopback

# Time to truncate - We need to keep the loop open
val1=$(fdisk -l $loopback | grep Units | sed -e 's/^Units .*= \([[:digit:]]\+\) bytes$/\1/')
val2=$(fdisk -l $loopback | grep '^/dev' | sed -e 's/\*//' | awk '{ print $3 }' | sort -n | tail -n 1)

# Now it's time to close the loop
losetup -d $loopback

#truncating using Byron's program.
/xenimages/truncate $image $((val1*val2))
exit
